@@ -18,7 +18,7 @@
 #ifndef _CONSUMER_H
 #define _CONSUMER_H
 
-#include <common/consumer.h>
+#include <common/consumer/consumer.h>
 #include <common/hashtable/hashtable.h>
 #include <lttng/lttng.h>
 #include <urcu/ref.h>
@@ -93,6 +93,11 @@ struct consumer_data {
 	int err_sock;
 	/* These two sockets uses the cmd_unix_sock_path. */
 	int cmd_sock;
+	/*
+	 * Write-end of the channel monitoring pipe to be passed to the
+	 * consumer.
+	 */
+	int channel_monitor_pipe;
 	/*
 	 * The metadata socket object is handled differently and only created
 	 * locally in this object thus it's the only reference available in the
@@ -153,6 +158,9 @@ struct consumer_output {
 	 * index. The relayd sockets are index with it on the consumer side.
 	 */
 	uint64_t net_seq_index;
+	/* Store the relay protocol in use if the session is remote. */
+	uint32_t relay_major_version;
+	uint32_t relay_minor_version;
 
 	/*
 	 * Subdirectory path name used for both local and network consumer.
@@ -211,6 +219,8 @@ int consumer_send_relayd_socket(struct consumer_socket *consumer_sock,
 		struct lttcomm_relayd_sock *rsock, struct consumer_output *consumer,
 		enum lttng_stream_type type, uint64_t session_id,
 		char *session_name, char *hostname, int session_live_timer);
+int consumer_send_channel_monitor_pipe(struct consumer_socket *consumer_sock,
+		int pipe);
 int consumer_send_destroy_relayd(struct consumer_socket *sock,
 		struct consumer_output *consumer);
 int consumer_recv_status_reply(struct consumer_socket *sock);
@@ -229,6 +239,7 @@ void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 		unsigned int switch_timer_interval,
 		unsigned int read_timer_interval,
 		unsigned int live_timer_interval,
+		unsigned int monitor_timer_interval,
 		int output,
 		int type,
 		uint64_t session_id,
@@ -245,6 +256,7 @@ void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 		uint64_t session_id_per_pid,
 		unsigned int monitor,
 		uint32_t ust_app_uid,
+		int64_t blocking_timeout,
 		const char *root_shm_path,
 		const char *shm_path);
 void consumer_init_stream_comm_msg(struct lttcomm_consumer_msg *msg,
@@ -270,7 +282,8 @@ void consumer_init_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 		uint64_t tracefile_size,
 		uint64_t tracefile_count,
 		unsigned int monitor,
-		unsigned int live_timer_interval);
+		unsigned int live_timer_interval,
+		unsigned int monitor_timer_interval);
 int consumer_is_data_pending(uint64_t session_id,
 		struct consumer_output *consumer);
 int consumer_close_metadata(struct consumer_socket *socket,
@@ -279,8 +292,13 @@ int consumer_setup_metadata(struct consumer_socket *socket,
 		uint64_t metadata_key);
 int consumer_push_metadata(struct consumer_socket *socket,
 		uint64_t metadata_key, char *metadata_str, size_t len,
-		size_t target_offset);
+		size_t target_offset, uint64_t version);
 int consumer_flush_channel(struct consumer_socket *socket, uint64_t key);
+int consumer_clear_quiescent_channel(struct consumer_socket *socket, uint64_t key);
+int consumer_get_discarded_events(uint64_t session_id, uint64_t channel_key,
+		struct consumer_output *consumer, uint64_t *discarded);
+int consumer_get_lost_packets(uint64_t session_id, uint64_t channel_key,
+		struct consumer_output *consumer, uint64_t *lost);
 
 /* Snapshot command. */
 int consumer_snapshot_channel(struct consumer_socket *socket, uint64_t key,
