@@ -29,6 +29,7 @@
 #include "session.h"
 #include "ust-app.h"
 #include "version.h"
+#include "notification-thread.h"
 
 extern const char default_home_dir[],
 	default_tracing_group[],
@@ -38,6 +39,8 @@ extern const char default_home_dir[],
 /* Set in main.c at boot time of the daemon */
 extern int kernel_tracer_fd;
 
+extern struct notification_thread_handle *notification_thread_handle;
+
 /*
  * This contains extra data needed for processing a command received by the
  * session daemon from the lttng client.
@@ -58,7 +61,7 @@ struct ust_command {
 };
 
 /*
- * Queue used to enqueue UST registration request (ust_commant) and protected
+ * Queue used to enqueue UST registration request (ust_command) and synchronized
  * by a futex with a scheme N wakers / 1 waiters. See futex.c/.h
  */
 struct ust_cmd_queue {
@@ -121,6 +124,8 @@ extern const char * const config_section_name;
 /* Is this daemon root or not. */
 extern int is_root;
 
+extern const char *tracing_group_name;
+
 int sessiond_check_thread_quit_pipe(int fd, uint32_t events);
 int sessiond_set_thread_pollset(struct lttng_poll_event *events, size_t size);
 void sessiond_notify_ready(void);
