@@ -528,7 +528,7 @@ int utils_create_lock_file(const char *filepath)
 		S_IRGRP | S_IWGRP);
 	if (fd < 0) {
 		PERROR("open lock file %s", filepath);
-		ret = -1;
+		fd = -1;
 		goto error;
 	}
 
@@ -1017,6 +1017,59 @@ static inline unsigned int fls_u32(uint32_t x)
 #define HAS_FLS_U32
 #endif
 
+#if defined(__x86_64)
+static inline
+unsigned int fls_u64(uint64_t x)
+{
+	long r;
+
+	asm("bsrq %1,%0\n\t"
+	    "jnz 1f\n\t"
+	    "movq $-1,%0\n\t"
+	    "1:\n\t"
+	    : "=r" (r) : "rm" (x));
+	return r + 1;
+}
+#define HAS_FLS_U64
+#endif
+
+#ifndef HAS_FLS_U64
+static __attribute__((unused))
+unsigned int fls_u64(uint64_t x)
+{
+	unsigned int r = 64;
+
+	if (!x)
+		return 0;
+
+	if (!(x & 0xFFFFFFFF00000000ULL)) {
+		x <<= 32;
+		r -= 32;
+	}
+	if (!(x & 0xFFFF000000000000ULL)) {
+		x <<= 16;
+		r -= 16;
+	}
+	if (!(x & 0xFF00000000000000ULL)) {
+		x <<= 8;
+		r -= 8;
+	}
+	if (!(x & 0xF000000000000000ULL)) {
+		x <<= 4;
+		r -= 4;
+	}
+	if (!(x & 0xC000000000000000ULL)) {
+		x <<= 2;
+		r -= 2;
+	}
+	if (!(x & 0x8000000000000000ULL)) {
+		x <<= 1;
+		r -= 1;
+	}
+	return r;
+}
+#endif
+
 #ifndef HAS_FLS_U32
 static __attribute__((unused)) unsigned int fls_u32(uint32_t x)
 {
@@ -1063,6 +1116,20 @@ int utils_get_count_order_u32(uint32_t x)
 	return fls_u32(x - 1);
 }
 
+/*
+ * Return the minimum order for which x <= (1UL << order).
+ * Return -1 if x is 0.
+ */
+LTTNG_HIDDEN
+int utils_get_count_order_u64(uint64_t x)
+{
+	if (!x) {
+		return -1;
+	}
+
+	return fls_u64(x - 1);
+}
+
 /**
  * Obtain the value of LTTNG_HOME environment variable, if exists.
  * Otherwise returns the value of HOME.
@@ -1362,11 +1429,17 @@ static const char *get_man_bin_path(void)
 }
 
 LTTNG_HIDDEN
-int utils_show_man_page(int section, const char *page_name)
+int utils_show_help(int section, const char *page_name,
+		const char *help_msg)
 {
 	char section_string[8];
 	const char *man_bin_path = get_man_bin_path();
-	int ret;
+	int ret = 0;
+
+	if (help_msg) {
+		printf("%s", help_msg);
+		goto end;
+	}
 
 	/* Section integer -> section string */
 	ret = sprintf(section_string, "%d", section);
@@ -1375,11 +1448,13 @@ int utils_show_man_page(int section, const char *page_name)
 	/*
 	 * Execute man pager.
 	 *
-	 * We provide --manpath to man here because LTTng-tools can
+	 * We provide -M to man here because LTTng-tools can
 	 * be installed outside /usr, in which case its man pages are
 	 * not located in the default /usr/share/man directory.
 	 */
-	ret = execlp(man_bin_path, "man", "--manpath", MANPATH,
+	ret = execlp(man_bin_path, "man", "-M", MANPATH,
 		section_string, page_name, NULL);
+
+end:
 	return ret;
 }
